#include<stdio.h>
#include "vilog.cpp" 
#include "grade.h" 

grade::grade ( void ) 
{ 
    pool_rate = 10; 
    radioactivity = 0.2; 
    cross_limit = 1.0;      
 
    ActualSize=0; 
    generation=0; 
    fitness_call=0; 
    PoolSize=0; 
    SelectedSize=0; 
    Force=NULL; 
    CH=NULL; 
    bsf=NULL; 
    bsf_birth=0; 
    btg=NULL; 
    F=NULL; 
} 
 
grade::~grade ( void ) 
{ 
} 

void grade::clear ( void ) 
{ 
    clear_pool (); 
} 
 
void grade::new_point ( double *x ) 
{ 
    int i; 
 
    for ( i=0; i<F->Dim; i++ ) 
        x[i] = random_double ( F->Domain[i][0], F->Domain[i][1] ); 
} 
 
void grade::create ( void ) 
{ 
    PoolSize = 2*pool_rate*F->Dim; 
    SelectedSize = pool_rate*F->Dim; 
 
    int i; 
    Force = new double [PoolSize]; 
    CH = new p_double [PoolSize]; 
    origine = new int [PoolSize]; 
    for ( i=0; i<PoolSize; i++ ) 
    { 
        CH[i] = new double [F->Dim]; 
        origine[i] = 0; 
    } 
    bsf = new double [F->Dim]; 
} 
 
void grade::EVALUATE_GENERATION ( int start ) 
{ 
    int i, k; 

    for ( i=start*SelectedSize; i<ActualSize; i++ ) 
    { 
        if ( F->Return_to_domain ) 
        { 
            for ( k=0; k<F->Dim; k++ ) 
            { 
                if ( CH[i][k]<F->Domain[k][0] ) CH[i][k]=F->Domain[k][0]; 
                if ( CH[i][k]>F->Domain[k][1] ) CH[i][k]=F->Domain[k][1]; 
            } 
        } 
        Force[i] = F->value ( CH[i] ); 
        fitness_call++; 
        if ( Force[i]>btg_value ) 
        { 
            btg_value = Force[i]; 
            btg=CH[i]; 
            btg_or = origine[i]; 
        } 
    } 

    if ( btg_value>bsf_value ) 
    { 
        memcpy ( bsf, btg, F->Dim*sizeof ( double ) ); 
        bsf_value = btg_value; 
        bsf_birth = fitness_call; 
        F->evaluate( bsf, fitness_call, bsf_value ); 
    } 
}	 
 
void grade::SELECT ( void ) 
{ 
    double *h; 
    int i1, i2, dead, last; 
 
    while ( ActualSize > SelectedSize ) 
    { 
        i1 = random_int ( 0,ActualSize-1 ); 
        i2 = random_int ( 1,ActualSize-1 ); 
        if ( i1==i2 ) i2--; 
        if ( Force[i1] >= Force[i2] ) dead = i2; 
        else dead = i1; 
        last = ActualSize-1; 
        h = CH[last]; 
        CH[last] = CH[dead]; 
        CH[dead] = h; 
        if ( btg==CH[last] ) btg=CH[dead]; 
        Force[dead] = Force[last]; 
        origine[dead] = origine[last]; 
        ActualSize--; 
    } 
}	 
 
void grade::MUTATE ( void ) 
{ 
  double p, *x, mutation_rate; 
    int i, j, index; 
 
    x = new double [F->Dim]; 
    for ( i=0; i<SelectedSize; i++ ) 
    { 
        if ( ActualSize == PoolSize ) break; 
        p = random_double ( 0.0, 1.0 ); 
        if ( p<=radioactivity ) 
        { 
            index = random_int ( 0,SelectedSize-1 ); 
            mutation_rate=random_double ( 0.0, 1.0 ); 
            new_point( x ); 
            for ( j=0;  j<F->Dim; j++ ) 
            { 
                CH[ActualSize][j] = CH[index][j]+mutation_rate*( x[j]-CH[index][j] ); 
            } 
            origine[ActualSize] = 2; 
            ActualSize++; 
        } 
    } 
    delete [] x; 
} 
 
void grade::CROSS ( void ) 
{ 
    int i1,i2,i3,j; 
    double cross_rate; 
    while ( ActualSize < PoolSize ) 
    { 
      i1 = random_int( 0,SelectedSize-1 ); 
      i2 = random_int( 1,SelectedSize-1 ); 
      if ( i1==i2 ) i2--; 
      i3 = i2; 
      cross_rate = random_double( 0.0, cross_limit ); 
      if ( Force[i1]>Force[i2] ) i3 = i1; 
      for ( j=0 ; j<F->Dim ; j++ ) 
	CH[ActualSize][j] = CH[i3][j]+cross_rate*( CH[i2][j]-CH[i1][j] ) ; 
      origine[ActualSize] = 1; 
      ActualSize++; 
    } 
} 
 
void grade::FIRST_GENERATION ( void) 
{ 
  int i; 
  ActualSize = PoolSize; 
  bsf_value = MinDouble ; 
  btg_value = MinDouble ; 
  for ( i=0; i<ActualSize; i++ ) new_point ( CH[i] ); 
  new_point ( bsf ); 
  btg = NULL; 
  EVALUATE_GENERATION ( 0 ); 
  SELECT (); 
  vilog_news ();
}	 
 
void grade::clear_pool ( void ) 
{ 
    int i; 
    for ( i=0; i<PoolSize; i++ ) if ( CH[i] ) delete [] CH[i]; 
    if ( CH ) delete [] CH; 
    if ( Force ) delete [] Force; 
    if ( bsf ) delete [] bsf; 
    if ( origine ) delete [] origine; 
} 
 
// -------------------------------------- STOPPING CRITERIA ------------------------------------------------ 
 
int grade::to_continue () 
{ 
    if ( fitness_call > fitness_calls_limit ) return 0; 
    if ( F->optimum ) if ( *F->optimum-bsf_value<=F->precision ) return 0; 
    return 1; 
} 
 
// -------------------------------------- MAIN ------------------------------------------------------------- 
 
void grade::run ( void ) 
{ 
  int i;
  int stop=0, monsters; 

  bsf_value=MinDouble; 
  btg_value=MinDouble;
  sbtg=MinDouble;
  FIRST_GENERATION (); 

  while ( !stop ) { 
    MUTATE (); 
    monsters=ActualSize;
    CROSS (); 
    EVALUATE_GENERATION ( 1 ); 
    SELECT (); 
    if ( !vilog_news () ) stop=1; 
    if ( !to_continue () ) stop=1; 
    generation++; 
  } 
  
  vilog_results (); 
  vilog_make_out (); 
  
  F->evaluate ( bsf ); 
} 

/*
author: Anicka Kucerova, Czech Technical University in Prague
version 0.0.1
19 Mar 2009
*/
